#include <stdlib.h>
#include <sdl/sdl.h>
#include "archdep.h"
#include "synthgui.h"
#include "gui.h"

extern SynthGUI *synthgui;

static SDL_Surface *surf;
static int base;

extern void triggerNote(int note);
extern void releaseNote(void);

struct {
	
	unsigned char r, g, b;
} palette[256];

static SDL_Color rpalette[256];

void initVideo(void) {

  base = 2;

  if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0) {
	  
    fprintf(stderr, "Couldn't initialize SDL\n");
    exit(1);
  }
  atexit(SDL_Quit);

  surf = SDL_SetVideoMode(320, 240, 8,
                          (SDL_GetVideoInfo()->hw_available ?
                             SDL_HWSURFACE : SDL_SWSURFACE) | 
                               SDL_HWPALETTE | SDL_DOUBLEBUF
                         );
}

void initAudio(void (*audio_callback)(void *, unsigned char *, int)) {

  SDL_AudioSpec spec;

  spec.freq = 44100;
  spec.format = AUDIO_S16;
  spec.channels = 1;
  spec.samples = BUFLEN;
  spec.callback = audio_callback;
  spec.userdata = 0;

  if (SDL_OpenAudio(&spec, 0) < 0) {

    fprintf(stderr, "Couldn't start audio: %s\n", SDL_GetError());
    exit(1);
  }

	SDL_PauseAudio(0);
}

void stopAudio(void) {
	
	SDL_PauseAudio(1);
	SDL_CloseAudio();
}

void refreshScreen(void) {
	
	SDL_UpdateRect(surf, 0, 0, 0, 0);
}

void commitPal(int brightness) {
	
	int i;

	for (i = 0; i < 256; i++) {
		
		int r, g, b;
		
		r = ((int)palette[i].r * brightness) / 128;
		g = ((int)palette[i].g * brightness) / 128;
		b = ((int)palette[i].b * brightness) / 128;

		if (r > 255) r = 255;
		if (g > 255) g = 255;
		if (b > 255) b = 255;

		rpalette[i].r = r;
		rpalette[i].g = g;
		rpalette[i].b = b;
	}

	SDL_SetPalette(surf, SDL_LOGPAL | SDL_PHYSPAL, rpalette, 0, 256);
}

void setPal(int i, unsigned char *c) {
	
	palette[i].r = c[0];
	palette[i].g = c[1];
	palette[i].b = c[2];
}

unsigned char *getPixels(void) {
	
	return (unsigned char*)surf->pixels;
}

void moveCursor(int x, int y) {
	
	SDL_WarpMouse(x, y);
}

void showCursor(void) {

  SDL_ShowCursor(SDL_ENABLE);
}

void hideCursor(void) {

  SDL_ShowCursor(SDL_DISABLE);
}

void waitKey(void) {
	
	SDL_Event event;
	
	while (1) {
		
		if (SDL_PollEvent(&event)) {
			
			if (event.type == SDL_KEYDOWN)
				return;
		}
	}
}

int pollmachine(void) {
	
  SDL_Event event;
	SDL_Delay(1);
  while (SDL_PollEvent(&event) != 0) {
	  
    switch (event.type) {
	    
      case SDL_QUIT:
        return -1;
        break;
      case SDL_MOUSEMOTION:
      	moveMouseEvent(event.motion.x, event.motion.y);
      	break;
      case SDL_MOUSEBUTTONDOWN:
      	if (event.button.button == SDL_BUTTON_LEFT)
      		pressMouseButtonEvent();
      	break;
			case SDL_MOUSEBUTTONUP:
				if (event.button.button == SDL_BUTTON_LEFT)
					releaseMouseButtonEvent();
				break;
				
			case SDL_KEYDOWN:
			case SDL_KEYUP: {

				int t = event.type == SDL_KEYDOWN;
				
				switch (event.key.keysym.sym) {
					
					case SDLK_ESCAPE:
						return -1;
						break;
					
					case SDLK_F1:
						if (t) base = 0;
						break;
					case SDLK_F2:
						if (t) base = 1;
						break;
					case SDLK_F3:
						if (t) base = 2;
						break;
					case SDLK_F4:
						if (t) base = 3;
						break;
					case SDLK_F5:
						if (t) base = 4;
						break;
					case SDLK_F6:
						if (t) base = 5;
						break;
					case SDLK_F7:
						if (t) base = 6;
						break;
					case SDLK_F8:
						if (t) base = 7;
						break;
					case SDLK_F9:
						if (t) base = 8;
						break;
					case SDLK_F10:
						if (t) base = 9;
						break;

					case SDLK_F11:
						synthgui->getSynth()->loadProgram("00.dat");
						synthgui->update();
						break;

					case SDLK_F12:
						synthgui->getSynth()->saveProgram("00.dat");
						break;

					case SDLK_q:	/* C-1 */
						if (t)
							triggerNote(0 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_2:	/* C#1 */
						if (t)
							triggerNote(1 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_w:	/* D-1 */
						if (t)
							triggerNote(2 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_3:	/* D#1 */
						if (t)
							triggerNote(3 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_e:	/* E-1 */
						if (t)
							triggerNote(4 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_r:	/* F-1 */
						if (t)
							triggerNote(5 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_5:	/* F#1 */
						if (t)
							triggerNote(6 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_t:	/* G-1 */
						if (t)
							triggerNote(7 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_6:	/* G#1 */
						if (t)
							triggerNote(8 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_y:	/* A-1 */
						if (t)
							triggerNote(9 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_7:	/* A#1 */
						if (t)
							triggerNote(10 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_u:	/* B-1 */
						if (t)
							triggerNote(11 + ((base + 1) * 12));
						else
							releaseNote();
						break;
					case SDLK_i:	/* C-2 */
						if (t)
							triggerNote(12 + ((base + 1) * 12));
						else
							releaseNote();
						break;


					case SDLK_z:	/* C-0 */
						if (t)
							triggerNote(0 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_s:	/* C#0 */
						if (t)
							triggerNote(1 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_x:	/* D-0 */
						if (t)
							triggerNote(2 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_d:	/* D#0 */
						if (t)
							triggerNote(3 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_c:	/* E-0 */
						if (t)
							triggerNote(4 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_v:	/* F-0 */
						if (t)
							triggerNote(5 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_g:	/* F#0 */
						if (t)
							triggerNote(6 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_b:	/* G-0 */
						if (t)
							triggerNote(7 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_h:	/* G#0 */
						if (t)
							triggerNote(8 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_n:	/* A-0 */
						if (t)
							triggerNote(9 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_j:	/* A#0 */
						if (t)
							triggerNote(10 + (base * 12));
						else
							releaseNote();
						break;
					case SDLK_m:	/* B-0 */
						if (t)
							triggerNote(11 + (base * 12));
						else
							releaseNote();
						break;


					default:
						break;
				}

				break;
			}
				
      default:
        break;
    }
  }
  
  return SYNTH_OK;
}

void exitApp(void) {
	
	exit(0);
}
