#include "VSTEditor.h"
#include "VSTFrame.h"
#include "VSTPlugin.h"
#include "vstinfo.h"

VSTEditor::VSTEditor(AudioEffect *effect) : AEffGUIEditor(effect) {
	
	bOpened = false;
	synthgui = new SynthGUI(VSTWIDTH, VSTHEIGHT);
}

VSTEditor::~VSTEditor() {
	
	delete synthgui;
}

SynthGUI *VSTEditor::getSynthGUI() {
	
	return synthgui;
}

void VSTEditor::setPalette() {
	
	int i;
	GUIColor palette[256];
	
	synthgui->getPalette(palette);
	
	for (i = 0; i < 256; i++) {
		
		hBackground->setPaletteEntry(i, palette[i].r, palette[i].g, palette[i].b);
	}
}

void VSTEditor::mouseMove(CFrame *frame, CDrawContext *pContext, int x, int y) {
	
	synthgui->mouseMoved(x, y);
	if (synthgui->mustDraw()) {
		
		synthgui->draw(hBackground->getSurface());
		hBackground->commitBitmap();
		frame->draw(pContext);
	}
}

void VSTEditor::mouseClick(CFrame *frame, CDrawContext *pContext) {
	
	synthgui->mousePressed();
	if (synthgui->mustDraw()) {
		
		synthgui->draw(hBackground->getSurface());
		hBackground->commitBitmap();
		frame->draw(pContext);
	}
}

void VSTEditor::mouseRelease(CFrame *frame, CDrawContext *pContext) {
	
	synthgui->mouseReleased();
	if (synthgui->mustDraw()) {
		
		synthgui->draw(hBackground->getSurface());
		hBackground->commitBitmap();
		frame->draw(pContext);
	}
}

long VSTEditor::open(void *ptr) {
	
	// !!! always call this !!!
	AEffGUIEditor::open(ptr);

	CRect size(0, 0, VSTWIDTH, VSTHEIGHT);
	frame = new VSTFrame(size, ptr, this);
	rect.left   = 0;
	rect.top    = 0;
	rect.right  = VSTWIDTH;
	rect.bottom = VSTHEIGHT;

	hBackground = new CSurface(*frame, VSTWIDTH, VSTHEIGHT);
	setPalette();
	synthgui->forceDraw();
	synthgui->draw(hBackground->getSurface());
	hBackground->commitBitmap();
	frame->setBackground(hBackground);

	bOpened = true;
	
	return true;
}

void VSTEditor::close() {

	bOpened = false;
	delete frame;
	frame = 0;
}

void VSTEditor::setParameter (long index, float value) {
	
	if (!bOpened)
		return;
}

void VSTEditor::valueChanged(CDrawContext* context, CControl* control) {

}

void VSTEditor::redraw() {

	if (bOpened) {

		synthgui->draw(hBackground->getSurface());
		hBackground->commitBitmap();
		frame->draw();
	}
}
