//-------------------------------------------------------------------------------------------------------
// VST Plug-Ins SDK
// Example AGain (VST 1.0)
// Stereo plugin which applies a Gain [-oo, 0dB]
//  2003, Steinberg Media Technologies, All Rights Reserved
//-------------------------------------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include "VSTPlugin.h"
#include "VSTEditor.h"

#include "vstinfo.h"

//-------------------------------------------------------------------------------------------------------
VSTPlugin::VSTPlugin (audioMasterCallback audioMaster)
	: AudioEffectX (audioMaster, VSTNUMPROGRAMS, VSTNUMPARAMETERS)
{
	fGain = 1.;				// default to 0 dB
	setNumInputs(0);		// stereo in
	setNumOutputs(1);		// stereo out
#if VSTI == 1
	isSynth();
#endif
	setUniqueID(VSTID);	// identify
	canMono();				// makes sense to feed both inputs with the same signal
	canProcessReplacing();	// supports both accumulating and replacing output
	strcpy (programName, VSTNAME);	// default program name

	buffer = 0;
	editor = new VSTEditor(this);
}

//-------------------------------------------------------------------------------------------------------
VSTPlugin::~VSTPlugin ()
{
	free(buffer);
}

//-------------------------------------------------------------------------------------------------------
void VSTPlugin::setProgramName (char *name)
{
	strcpy(programName, name);
}

//-----------------------------------------------------------------------------------------
void VSTPlugin::getProgramName (char *name)
{
	sprintf(name, "Prog%02d", curProgram);
}

//-----------------------------------------------------------------------------------------
void VSTPlugin::setParameter (long index, float value)
{
}

//-----------------------------------------------------------------------------------------
float VSTPlugin::getParameter (long index)
{
	return 0.0f;
}

//-----------------------------------------------------------------------------------------
void VSTPlugin::getParameterName (long index, char *label)
{
	strcpy (label, "Gain");
}

//-----------------------------------------------------------------------------------------
void VSTPlugin::getParameterDisplay (long index, char *text)
{
	dB2string (fGain, text);
}

//-----------------------------------------------------------------------------------------
void VSTPlugin::getParameterLabel(long index, char *label)
{
	strcpy (label, "dB");
}

//------------------------------------------------------------------------
bool VSTPlugin::getEffectName (char* name)
{
	strcpy (name, "Gain");
	return true;
}

//------------------------------------------------------------------------
bool VSTPlugin::getProductString (char* text)
{
	strcpy (text, "Gain");
	return true;
}

//------------------------------------------------------------------------
bool VSTPlugin::getVendorString (char* text)
{
	strcpy (text, "Steinberg Media Technologies");
	return true;
}

//-----------------------------------------------------------------------------------------
void VSTPlugin::process (float **inputs, float **outputs, long sampleFrames)
{
    float *out1 = outputs[0];
    
		((VSTEditor*)editor)->getSynthGUI()->getSynth()->process(buffer, sampleFrames);    

    while (--sampleFrames >= 0)
    {
        (*out1++) += (*buffer++);
    }
}

//-----------------------------------------------------------------------------------------
void VSTPlugin::processReplacing (float **inputs, float **outputs, long sampleFrames)
{
    float *out1 = outputs[0];

    ((VSTEditor*)editor)->getSynthGUI()->getSynth()->process(out1, sampleFrames);
}

void VSTPlugin::resume(void) {
	
	wantEvents();
}

long VSTPlugin::processEvents(VstEvents* events) {
	
	long i;
	
	for (i = 0; i < events->numEvents; i++) {
		
		VstEvent *event = events->events[i];
		VstMidiEvent *midi = (VstMidiEvent*)event;
		char *midiData;
		long status, note, velocity;
		
		if (event->type != kVstMidiType)
			continue;
		
		midiData = midi->midiData;
		status = midiData[0] & 0xf0;
		note = midiData[1] & 0x7f;
		velocity = midiData[2] & 0x7f;
		
		if (status == 0x80)
			((VSTEditor*)editor)->getSynthGUI()->getSynth()->releaseNote();
		else
		if (status == 0x90)
			((VSTEditor*)editor)->getSynthGUI()->getSynth()->triggerNote(note);
	}
	
	return 1;
}

long VSTPlugin::getProgram () {
	
	return ((VSTEditor*)editor)->getSynthGUI()->getProgram();
}

void VSTPlugin::setProgram (long program) {

	AudioEffect::setProgram(program);	
	((VSTEditor*)editor)->getSynthGUI()->setProgram(program);
	((VSTEditor*)editor)->redraw();
}

void  VSTPlugin::setBlockSize(long blockSize) {

	if (buffer)
		free(buffer);
	
	buffer = (float*)malloc(sizeof(float) * blockSize);	
}
